@@ -2,9 +2,8 @@ import numpy as np
 from sklearn.metrics import confusion_matrix
 from .fairness import Fairness
 from .fairness_metrics import FairnessMetrics
-from .performance_metrics import PerformanceMetrics
+from ..performance_metrics import PerformanceMetrics
 from ..custom.newmetric import *
-from ..custom.newmetric_child import *
 from ..utility import check_datatype, check_value
 from ..config.constants import Constants
 from ..ErrorCatcher import VeritasError
@@ -15,13 +14,13 @@ class CreditScoring(Fairness):
     Class Attributes
     ------------------
     _model_type_to_metric_lookup: dictionary
-                Used to associate the model type (key) with the metric type & expected size of positive and negative labels (value).
-                e.g. {“rejection”: (“classification”, 2), “uplift”: (“uplift”, 4), “a_new_type”: (“regression”, -1)}
+                Used to associate the model type (key) with the metric type, expected size of positive and negative labels (value) & length of model_params respectively.
+                e.g. {“rejection”: (“classification”, 2, 1), “uplift”: (“uplift”, 4, 1), “a_new_type”: (“regression”, -1, 1)}
     """
 
     _model_type_to_metric_lookup = {"default": ("classification", 2, 1)}
 
-    def __init__(self, model_params, fair_threshold, perf_metric_name = "balanced_acc", fair_metric_name = "auto", fair_concern = "eligible", fair_priority = "benefit", fair_impact = "normal", fair_neutral_tolerance = 0.001, num_applicants = None, base_default_rate = None, fairness_metric_value_input = {}):
+    def __init__(self, model_params, fair_threshold, perf_metric_name = "balanced_acc", fair_metric_name = "auto", fair_concern = "eligible", fair_priority = "benefit", fair_impact = "normal", num_applicants = None, base_default_rate = None, fairness_metric_value_input = {}):
         """
         Parameters
         ----------
@@ -31,7 +30,7 @@ class CreditScoring(Fairness):
         fair_threshold: int or float
                 Value between 0 and 100. If a float between 0 and 1 (not inclusive) is provided, it is converted to a percentage and the p % rule is used to calculate the fairness threshold value.
 
-        perf_metric_name: string, default = ‘balanced_acc’
+        perf_metric_name: string, default = 'balanced_acc'
                 Name of the primary performance metric to be used for computations in the evaluate() and/or compile() functions.
 
         fair_metric_name : string, default="auto"
@@ -67,7 +66,7 @@ class CreditScoring(Fairness):
                 If both the base_default_rate & num_rejected_applicants are not None, the flag will be set to True.
                 e.g. {"gender": True, "race": False, "age": True}
 
-        _use_case_metrics: dictionary of lists, default="None"
+        _use_case_metrics: dictionary of lists, default=None
                 Contains all the performance & fairness metrics for credit scoring.
                 e.g. {"fair ": ["fnr_parity", ...], "perf": ["balanced_accuracy, ..."]}
                 Dynamically assigned during initialisation by using the _metric_group_map in Fairness/Performance Metrics class and the _model_type_to_metric above.
@@ -104,14 +103,12 @@ class CreditScoring(Fairness):
 
         err : object
                 VeritasError object
-
-        
-
         """
         super().__init__(model_params)
         self.perf_metric_name = perf_metric_name
         self.fair_threshold = fair_threshold
-        self.fair_neutral_tolerance = fair_neutral_tolerance
+        self.fair_threshold_input = fair_threshold
+        self.fair_neutral_tolerance = Constants().fair_neutral_tolerance 
         self.fair_concern = fair_concern
         self.fair_priority = fair_priority
         self.fair_impact = fair_impact
@@ -124,6 +121,7 @@ class CreditScoring(Fairness):
         #     self.err.push(ch_model[0][0], var_name=ch_model[0][1], given=ch_model[0][2], expected=ch_model[0][3])
 
         self.fair_metric_name = fair_metric_name
+        self.fair_metric_input = fair_metric_name
 
         self._select_fairness_metric_name()
 
@@ -148,11 +146,11 @@ class CreditScoring(Fairness):
         self._input_validation_lookup = {
             "fair_threshold": [(float, int), (Constants().fair_threshold_low), Constants().fair_threshold_high],
             "fair_neutral_tolerance": [(float,),(Constants().fair_neutral_threshold_low), Constants().fair_neutral_threshold_high],
-            "perf_metric_name": [(str,), self._use_case_metrics["perf"]],
-            "fair_metric_name": [(str,), self._use_case_metrics["fair"]],
             "fair_concern": [(str,), ["eligible", "inclusive", "both"]],
             "fair_priority": [(str,), ["benefit", "harm"]],
             "fair_impact": [(str,), ["normal", "significant", "selective"]],
+            "perf_metric_name": [(str,), self._use_case_metrics["perf"]],
+            "fair_metric_name": [(str,), self._use_case_metrics["fair"]],
             "model_params":[(list,), None],
             "fairness_metric_value_input":[(dict,), None]}
         
@@ -166,11 +164,13 @@ class CreditScoring(Fairness):
            self._rejection_inference_flag[var] = False
 
         if self.spl_params['base_default_rate'] is not None and self.spl_params['num_applicants'] is not None:
-            for i in self.spl_params['base_default_rate']:
-                for j in self.spl_params['num_applicants']:
-                    if i == j:
-                        if len(self.spl_params['base_default_rate'][i]) == 2 and len(self.spl_params['num_applicants'][j]) == 2:
-                            self._rejection_inference_flag[i] = True
+            for var in self.model_params[0].p_var:
+            # for i in self.spl_params['base_default_rate']:
+            #     for j in self.spl_params['num_applicants']:
+            #         if i == j:
+                        # if len(self.spl_params['base_default_rate'][i]) == 2 and len(self.spl_params['num_applicants'][j]) == 2:
+                self._rejection_inference_flag[var] = True
+               
 
         # self.rejection_inference_flagging()
         # self.check_applicants_approvals()
@@ -321,7 +321,7 @@ class CreditScoring(Fairness):
         #check for model_params
         # val_errmsg = "\n The given length of model_params is {}. \n The expected length of model_params is {}."
         mp_given = len(self.model_params)
-        mp_expected = int(self._model_type_to_metric_lookup[self.model_params[0].model_type][1] / 2)
+        mp_expected = self._model_type_to_metric_lookup[self.model_params[0].model_type][2]
         if mp_given != mp_expected:
             self.err.push('length_error', var_name="model_params", given=str(mp_given), expected=str(mp_expected), function_name="_check_input")
             # raise ValueError(val_errmsg.format(str(mp_given), str(mp_expected)))
@@ -365,84 +365,108 @@ class CreditScoring(Fairness):
             if type(self.spl_params[i]) != dict and  type(self.spl_params[i]) != type(None):
                 # errMsg += errMsgFormat.format(str(i),type(self.spl_params[i]), "dict")
                 # raise ValueError(errMsg)
+                self._rejection_inference_flag = self._rejection_inference_flag.fromkeys(self._rejection_inference_flag, False)
                 self.err.push('value_error', var_name=str(i), given=type(self.spl_params[i]), expected="dict", function_name="_check_special_params")
+                self.err.pop()
+        # if not all(v is None for v in self.spl_params.values()):
+        #     print()
+        #     self._rejection_inference_flag = self._rejection_inference_flag.fromkeys(self._rejection_inference_flag, False)
+        #     self.err.push('value_error', var_name=str(i), given="None", expected="not None", function_name="_check_special_params")
 
-        num_applicants_type = (int, float, type(None))
-        base_default_rate_type = (float, type(None))
+        if self.spl_params['base_default_rate'] is not None:
+            for var in self.model_params[0].p_var:
+                if self._rejection_inference_flag[var] == True:
+                    if var not in self.spl_params['base_default_rate'].keys():
+                        self._rejection_inference_flag[var] = False 
+                        self.err.push('value_error', var_name='base_default_rate', given="values for " + str(var), expected="values for " + str( self.model_params[0].p_var), function_name="_check_special_params")
+                        
+        #check for length of num_applicants
+        # num_errMsg = "\n Length of num_applicants should be equal to 2. Given length for {} input is {}"
+        if self.spl_params['num_applicants'] is not None:
+            for var in self.model_params[0].p_var:
+                if self._rejection_inference_flag[var] == True:
+                    if var not in self.spl_params['num_applicants'].keys():
+                        self._rejection_inference_flag[var] = False 
+                        self.err.push('value_error', var_name='num_applicants', given="values for " + str(var), expected="values for " + str( self.model_params[0].p_var), function_name="_check_special_params")
+        # self.err.pop()
+        num_applicants_type = (int, float,)
+        base_default_rate_type = (float,)
         # if self.spl_params["num_applicants"] == None:
         #     self.err.push('type_error', var_name='num_applicants',
         #                     given="None",
         #                     expected=str(num_applicants_type), function_name="_check_input")
-        try:
+        if self.spl_params['num_applicants'] is not None:
             for i in self.spl_params["num_applicants"]:
-                for j in range(len(self.spl_params["num_applicants"].get(i))):
-                    if type(self.spl_params["num_applicants"].get(i)[j]) in num_applicants_type:
-                        if type(self.spl_params["num_applicants"].get(i)[j]) == float:
-                            self.spl_params["num_applicants"].get(i)[j] = int(self.spl_params["num_applicants"].get(i)[j])
-                        elif type(self.spl_params["num_applicants"].get(i)[j]) == type(None):
-                            continue
-                    else:
-                        self.err.push('type_error', var_name='num_applicants',
-                                    given=str(type(self.spl_params["num_applicants"].get(i)[j])),
-                                    expected=str(num_applicants_type), function_name="_check_special_params")
-        except TypeError:
-            pass
-        
+                if self._rejection_inference_flag[i] == True :
+                    for j in range(len(self.spl_params["num_applicants"].get(i))):
+                        if type(self.spl_params["num_applicants"].get(i)[j]) in num_applicants_type:
+                            if type(self.spl_params["num_applicants"].get(i)[j]) == float:
+                                self.spl_params["num_applicants"].get(i)[j] = int(self.spl_params["num_applicants"].get(i)[j])
+                        else:
+                            self._rejection_inference_flag[i] = False 
+                            self.err.push('type_error', var_name='num_applicants',
+                                        given=str(type(self.spl_params["num_applicants"].get(i)[j])),
+                                        expected=str(num_applicants_type), function_name="_check_special_params")
+
         # if self.spl_params["base_default_rate"] == None:
         #     self.err.push('type_error', var_name='base_default_rate',
         #                     given="None",
         #                     expected=str(base_default_rate_type), function_name="_check_input")
-        try:
+        if self.spl_params['base_default_rate'] is not None:
             for i in self.spl_params["base_default_rate"]:
-                for j in range(len(self.spl_params["base_default_rate"].get(i))):
-                    if type(self.spl_params["base_default_rate"].get(i)[j]) not in base_default_rate_type:
-                        self.err.push('type_error', var_name='base_default_rate',
-                                    given=str(type(self.spl_params["base_default_rate"].get(i)[j])),
-                                    expected=str(base_default_rate_type), function_name="_check_special_params")
+                if self._rejection_inference_flag[i] == True :
+                    for j in range(len(self.spl_params["base_default_rate"].get(i))):
+                        if type(self.spl_params["base_default_rate"].get(i)[j]) not in base_default_rate_type:
+                            self._rejection_inference_flag[i] = False 
+                            self.err.push('type_error', var_name='base_default_rate',
+                                        given=str(type(self.spl_params["base_default_rate"].get(i)[j])),
+                                        expected=str(base_default_rate_type), function_name="_check_special_params")
     
-        except TypeError:
-            pass
         # errMsg = "data value error"
         # errMsgFormat = "\n    {}: given value for {}[{}] is {}, expected in range {}"
-
+        #self.err.pop()
         num_applicants_range = (0, np.inf)
         base_default_rate_range = (0, 1)
         if self.spl_params["num_applicants"] is not None:
             for k, l in self.spl_params['num_applicants'].items() :
-                for m in l :
-                    try:
+                if self._rejection_inference_flag[k] == True :
+                    for m in l :
                         if m <= 0:
                             # errMsg += errMsgFormat.format('num_applicants ' , str(k) , str(l.index(m)), str(m), str(num_applicants_range))
                             # raise ValueError(errMsg)
+                            self._rejection_inference_flag[k] = False 
                             self.err.push('value_error', var_name='num_applicants for ' + str(k), given=str(m),
                                           expected=str(num_applicants_range), function_name="_check_special_params")
-                    except TypeError:
-                        pass
 
         if self.spl_params["base_default_rate"] is not None:
             for k, l in self.spl_params['base_default_rate'].items() :
-                for m in l :
-                    if  m < base_default_rate_range[0]  or m > base_default_rate_range[1] :
-                        # errMsg += errMsgFormat.format('base_default_rate ' , str(k) , str(l.index(m)), str(m), str(base_default_rate_range))
-                        # raise ValueError(errMsg)
-                        self.err.push('value_error', var_name='base_default_rate for ' + str(k), given=str(m),
-                                      expected=str(base_default_rate_range), function_name="_check_special_params")
+                if self._rejection_inference_flag[k] == True :
+                    for m in l :
+                        if  m < base_default_rate_range[0]  or m > base_default_rate_range[1] :
+                            # errMsg += errMsgFormat.format('base_default_rate ' , str(k) , str(l.index(m)), str(m), str(base_default_rate_range))
+                            # raise ValueError(errMsg)
+                            self._rejection_inference_flag[k] = False 
+                            self.err.push('value_error', var_name='base_default_rate for ' + str(k), given=str(m),
+                                        expected=str(base_default_rate_range), function_name="_check_special_params")
 
         #check for length of base_default_rate
         # bdr_errMsg = "\n Length of base_default_rate should be equal to 2. Given length for {} input is {}"
         if self.spl_params['base_default_rate'] is not None:
             for var in self.spl_params['base_default_rate']:
-                if len(self.spl_params['base_default_rate'][var]) != 2:
-                    # raise ValueError(bdr_errMsg.format(var, len(self.spl_params['base_default_rate'][var])))
-                    self.err.push('length_error', var_name='base_default_rate', given=len(self.spl_params['base_default_rate'][var]), expected='2', function_name="_check_special_params")
+                if self._rejection_inference_flag[var] == True :
+                    if len(self.spl_params['base_default_rate'][var]) != 2:
+                        self._rejection_inference_flag[var] = False 
+                        # raise ValueError(bdr_errMsg.format(var, len(self.spl_params['base_default_rate'][var])))
+                        self.err.push('length_error', var_name='base_default_rate', given=len(self.spl_params['base_default_rate'][var]), expected='2', function_name="_check_special_params")
         #check for length of num_applicants
         # num_errMsg = "\n Length of num_applicants should be equal to 2. Given length for {} input is {}"
         if self.spl_params['num_applicants'] is not None:
             for var in self.spl_params['num_applicants']:
-                if len(self.spl_params['num_applicants'][var]) != 2:
-                        # raise ValueError(num_errMsg.format(var, len(self.spl_params['num_applicants'][var])))
-                    self.err.push('length_error', var_name='num_applicants', given=len(self.spl_params['num_applicants'][var]), expected='2', function_name="_check_special_params")
-
+                if self._rejection_inference_flag[var] == True :
+                    if len(self.spl_params['num_applicants'][var]) != 2:
+                        self._rejection_inference_flag[var] = False 
+                            # raise ValueError(num_errMsg.format(var, len(self.spl_params['num_applicants'][var])))
+                        self.err.push('length_error', var_name='num_applicants', given=len(self.spl_params['num_applicants'][var]), expected='2', function_name="_check_special_params")
         #check for num of applicants
 
         val_lst = []
@@ -463,8 +487,10 @@ class CreditScoring(Fairness):
                 #     break
             try:
                 if sum(in_value[1]) != sum(val):
+                    print(sum(in_value[1]))
                         # raise ValueError(rej_msg)
                         # self.err.push('conflict_error', var_name_a='num_applicants', var_value_a=sum(val), var_value_b=sum(in_value[1]))
+                    self._rejection_inference_flag = self._rejection_inference_flag.fromkeys(self._rejection_inference_flag, False)
                     self.err.push('conflict_error', var_name_a="num_applicants", some_string="inconsistent values",
                                       value=val_lst, function_name="_check_special_params")
                     
@@ -481,38 +507,43 @@ class CreditScoring(Fairness):
             na_var = self.spl_params['num_applicants']
             if br_var is not None and na_var is not None:
                 for i in self.model_params[0].p_var:
-                    for j in br_var:
-                        for k in na_var:
-                            if i == j and i == k:
-                                #if len(br_var.get(i)) == 2 and len(na_var.get(i)) == 2:
-                                self.common_base_default_rate = (br_var.get(i)[0] * na_var.get(i)[0] + br_var.get(i)[1] * na_var.get(i)[1]) / (na_var.get(i)[0] + na_var.get(i)[1])
-                                check_cbdr[i] = self.common_base_default_rate
-                                # cbdr_errMsg += msg.format(i, check_cbdr[i])
+                    if self._rejection_inference_flag[i] == True :
+                        for j in br_var:
+                            for k in na_var:
+                                if i == j and i == k:
+                                    #if len(br_var.get(i)) == 2 and len(na_var.get(i)) == 2:
+                                    self.common_base_default_rate = (br_var.get(i)[0] * na_var.get(i)[0] + br_var.get(i)[1] * na_var.get(i)[1]) / (na_var.get(i)[0] + na_var.get(i)[1])
+                                    check_cbdr[i] = self.common_base_default_rate
+                                    # cbdr_errMsg += msg.format(i, check_cbdr[i])
                 br_value = next(iter(check_cbdr.items()))
                 for val in check_cbdr:
                     if round(br_value[1], 5) != round(check_cbdr[val], 5):
                         # raise ValueError(cbdr_errMsg)
+                        self._rejection_inference_flag = self._rejection_inference_flag.fromkeys(self._rejection_inference_flag, False)
                         self.err.push('conflict_error', var_name_a="Common base default rates", some_string="inconsistent values", value=[round(br_value[1], 5), round(check_cbdr[val], 5)], function_name="_check_special_params")
-        
+
         na_var = self.spl_params['num_applicants']
         exp_out = np.array(self.model_params[0].y_true)
         if na_var is not None:
             for i in self.model_params[0].p_var:
-                idx = self.feature_mask[i]
-                pri_grp = self.model_params[0].p_grp.get(i)[0]
-                features = self.model_params[0].x_test[i].unique().tolist()
-                for j in features:
-                    if j != pri_grp:
-                        unpri_grp = j
-    
-                if na_var.get(i)[0] < len(exp_out[idx]):
-                        # raise ValueError(num_errMSg.format(str(na_var.get(i)[0]), str(i), str(pri_grp), str(len(exp_out[idx])), str(i), str(pri_grp)))
-                    self.err.push('value_error_compare', var_name_a="Total number of applicants",
-                                      var_name_b="total number of approvals", function_name="_check_special_params")
-                elif na_var.get(i)[1] < len(exp_out[~idx]):
-                        # raise ValueError(num_errMSg.format(str(na_var.get(i)[1]), str(i), str(unpri_grp), str(len(exp_out[~idx])), str(i), str(unpri_grp)))
-                    self.err.push('value_error_compare', var_name_a="Total number of applicants",
-                                      var_name_b="total number of approvals", function_name="_check_special_params")    
+                if self._rejection_inference_flag[i] == True :
+                    idx = self.feature_mask[i]
+                    pri_grp = self.model_params[0].p_grp.get(i)[0]
+                    features = self.model_params[0].x_test[i].unique().tolist()
+                    for j in features:
+                        if j != pri_grp:
+                            unpri_grp = j
+        
+                    if na_var.get(i)[0] < len(exp_out[idx]):
+                            # raise ValueError(num_errMSg.format(str(na_var.get(i)[0]), str(i), str(pri_grp), str(len(exp_out[idx])), str(i), str(pri_grp)))
+                        self._rejection_inference_flag[i] = False 
+                        self.err.push('value_error_compare', var_name_a="Total number of applicants",
+                                        var_name_b="total number of approvals", function_name="_check_special_params")
+                    elif na_var.get(i)[1] < len(exp_out[~idx]):
+                            # raise ValueError(num_errMSg.format(str(na_var.get(i)[1]), str(i), str(unpri_grp), str(len(exp_out[~idx])), str(i), str(unpri_grp)))
+                        self._rejection_inference_flag[i] = False 
+                        self.err.push('value_error_compare', var_name_a="Total number of applicants",
+                                        var_name_b="total number of approvals", function_name="_check_special_params")    
         
         
         if self.model_params[0].y_pred is not None:
@@ -527,39 +558,43 @@ class CreditScoring(Fairness):
             if fn < 0 or tn < 0:
                     # raise ValueError(errMsg.format(group_default_rate, self.common_base_default_rate))
                     ## 
+                #print(fn,tn)
+                self._rejection_inference_flag = self._rejection_inference_flag.fromkeys(self._rejection_inference_flag, False)
                 self.err.push('conflict_error', var_name_a="base_default_rate and/or num_applicants", some_string="unrealistic_input",
                                 value='', function_name="_check_special_params")
 
             for curr_p_var in self.model_params[0].p_var :
-                mask = self.feature_mask[curr_p_var]
-                # use get conf matrix, not conf matrix 
-                # tn_p, fp_p, fn_p, tp_p = confusion_matrix(np.array(self.model_params[0].y_true)[mask], np.array(self.model_params[0].y_pred)[mask]).ravel()
-                tn_p, fp_p, fn_p, tp_p, tn_u, fp_u, fn_u, tp_u = self._get_confusion_matrix(y_true=self.model_params[0].y_true, y_pred=self.model_params[0].y_pred, sample_weight=self.model_params[0].sample_weight, curr_p_var = curr_p_var, feature_mask = self.feature_mask)
-                # tn_u, fp_u, fn_u, tp_u  = _get_confusion_matrix(np.array(self.model_params[0].y_true)[~mask], np.array(self.model_params[0].y_pred)[~mask]).ravel()
-                    # if self._rejection_inference_flag[curr_p_var] == True :
-                    #     fn_p = self.spl_params['num_applicants'][curr_p_var][0] * (1-self.spl_params['base_default_rate'][curr_p_var][0]) - tp_p
-                    #     tn_p = self.spl_params['num_applicants'][curr_p_var][0] * self.spl_params['base_default_rate'][curr_p_var][0] - fp_p
-                    #     fn_u= self.spl_params['num_applicants'][curr_p_var][1] * (1-self.spl_params['base_default_rate'][curr_p_var][1]) - tp_u
-                    #     tn_u = self.spl_params['num_applicants'][curr_p_var][1] * self.spl_params['base_default_rate'][curr_p_var][1] - fp_u
+                if self._rejection_inference_flag[curr_p_var] == True :
+                    mask = self.feature_mask[curr_p_var]
+                    # use get conf matrix, not conf matrix 
+                    # tn_p, fp_p, fn_p, tp_p = confusion_matrix(np.array(self.model_params[0].y_true)[mask], np.array(self.model_params[0].y_pred)[mask]).ravel()
+                    tn_p, fp_p, fn_p, tp_p, tn_u, fp_u, fn_u, tp_u = self._get_confusion_matrix(y_true=self.model_params[0].y_true, y_pred=self.model_params[0].y_pred, sample_weight=self.model_params[0].sample_weight, curr_p_var = curr_p_var, feature_mask = self.feature_mask)
+                    # tn_u, fp_u, fn_u, tp_u  = _get_confusion_matrix(np.array(self.model_params[0].y_true)[~mask], np.array(self.model_params[0].y_pred)[~mask]).ravel()
+                        # if self._rejection_inference_flag[curr_p_var] == True :
+                        #     fn_p = self.spl_params['num_applicants'][curr_p_var][0] * (1-self.spl_params['base_default_rate'][curr_p_var][0]) - tp_p
+                        #     tn_p = self.spl_params['num_applicants'][curr_p_var][0] * self.spl_params['base_default_rate'][curr_p_var][0] - fp_p
+                        #     fn_u= self.spl_params['num_applicants'][curr_p_var][1] * (1-self.spl_params['base_default_rate'][curr_p_var][1]) - tp_u
+                        #     tn_u = self.spl_params['num_applicants'][curr_p_var][1] * self.spl_params['base_default_rate'][curr_p_var][1] - fp_u
+            
+            
+                        # errMsg = "\n Group default rate  {:.3f} for {} - {} is higher than base default rate {:.3f} for {} - {}"
+
+                    pri_grp = self.model_params[0].p_grp.get(curr_p_var)[0]
+                    # unpri_grp = np.delete(self.model_params[0].x_test[curr_p_var].unique(), pri_grp)[0]
+
+                    if self.spl_params['num_applicants'] is not None:
+                        #print(self.spl_params['num_applicants'])
+                        group_default_rate_p = fp_p / self.spl_params['num_applicants'].get(curr_p_var)[0]
+                        group_default_rate_u = fp_u / self.spl_params['num_applicants'].get(curr_p_var)[1]
+                        if group_default_rate_p > self.spl_params['base_default_rate'].get(curr_p_var)[0]:
+                                # raise ValueError(errMsg.format(group_default_rate_p, str(curr_p_var), str(pri_grp), self.spl_params['base_default_rate'].get(curr_p_var)[0], str(curr_p_var), str(pri_grp)))
+                            self.err.push('conflict_error', var_name_a="base_default_rate", some_string="unrealistic_input",
+                                            value='', function_name="_check_special_params")
+                        if group_default_rate_u > self.spl_params['base_default_rate'].get(curr_p_var)[1]:
+                                # raise ValueError(errMsg.format(group_default_rate_u, str(curr_p_var), str(unpri_grp), self.spl_params['base_default_rate'].get(curr_p_var)[1], str(curr_p_var), str(unpri_grp)))
+                            self.err.push('conflict_error', var_name_a="base_default_rate", some_string="unrealistic_input",
+                                            value='', function_name="_check_special_params")
         
-        
-                    # errMsg = "\n Group default rate  {:.3f} for {} - {} is higher than base default rate {:.3f} for {} - {}"
-
-                pri_grp = self.model_params[0].p_grp.get(curr_p_var)[0]
-                # unpri_grp = np.delete(self.model_params[0].x_test[curr_p_var].unique(), pri_grp)[0]
-
-                if self.spl_params['num_applicants'] is not None:
-                    group_default_rate_p = fp_p / self.spl_params['num_applicants'].get(curr_p_var)[0]
-                    group_default_rate_u = fp_u / self.spl_params['num_applicants'].get(curr_p_var)[1]
-                    if group_default_rate_p > self.spl_params['base_default_rate'].get(curr_p_var)[0]:
-                            # raise ValueError(errMsg.format(group_default_rate_p, str(curr_p_var), str(pri_grp), self.spl_params['base_default_rate'].get(curr_p_var)[0], str(curr_p_var), str(pri_grp)))
-                        self.err.push('conflict_error', var_name_a="base_default_rate", some_string="unrealistic_input",
-                                        value='', function_name="_check_special_params")
-                    if group_default_rate_u > self.spl_params['base_default_rate'].get(curr_p_var)[1]:
-                            # raise ValueError(errMsg.format(group_default_rate_u, str(curr_p_var), str(unpri_grp), self.spl_params['base_default_rate'].get(curr_p_var)[1], str(curr_p_var), str(unpri_grp)))
-                        self.err.push('conflict_error', var_name_a="base_default_rate", some_string="unrealistic_input",
-                                        value='', function_name="_check_special_params")
-
         self.err.pop()
 
 
@@ -576,6 +611,20 @@ class CreditScoring(Fairness):
             self._fairness_tree()
         else :
             self.fair_metric_name
+        
+        # try:
+        if FairnessMetrics.map_fair_metric_to_group[self.fair_metric_name][4] == False:
+            err = VeritasError()
+            ratio_parity_metrics = []
+            for i,j in FairnessMetrics.map_fair_metric_to_group.items():
+                if j[1] == self._model_type_to_metric_lookup[self.model_params[0].model_type][0]:
+                    if j[2] == "ratio" or j[2] == "parity":
+                        ratio_parity_metrics.append(i)
+            err.push('value_error', var_name="fair_metric_name", given=self.fair_metric_name,  expected=ratio_parity_metrics, function_name="_select_fairness_metric_name")
+            err.pop()
+        # except:
+            # pass
+
 
     def _get_confusion_matrix(self, y_true, y_pred, sample_weight, curr_p_var = None, feature_mask = None, **kwargs):
         """
@@ -603,6 +652,7 @@ class CreditScoring(Fairness):
         -------
         Confusion matrix metrics based on privileged and unprivileged groups or a list of None if curr_p_var == None
         """
+        
         if self._model_type_to_metric_lookup[self.model_params[0].model_type][0] == "classification" :
 
             if 'y_true' in kwargs:
@@ -621,11 +671,11 @@ class CreditScoring(Fairness):
                     return [None] * 4
 
                 if sample_weight is None or len(rejection_inference_filter) > 0:
-                    tn, fp, fn, tp = confusion_matrix(y_true, y_pred).ravel()
+                    tn, fp, fn, tp = confusion_matrix(y_true=y_true, y_pred=y_pred).ravel()
                 else :
                         
                     # tn, fp, fn, tp = confusion_matrix(y_true, y_pred, sample_weight).ravel()
-                    tn, fp, fn, tp = confusion_matrix(y_true, y_pred, sample_weight = sample_weight).ravel()
+                    tn, fp, fn, tp = confusion_matrix(y_true=y_true, y_pred=y_pred, sample_weight=sample_weight).ravel()
                     
     
                 if len(rejection_inference_filter) > 0 :
@@ -642,11 +692,11 @@ class CreditScoring(Fairness):
                 mask = feature_mask[curr_p_var] 
                     
                 if sample_weight is None or len(rejection_inference_filter) > 0 :
-                    tn_p, fp_p, fn_p, tp_p = confusion_matrix(np.array(y_true)[mask], np.array(y_pred)[mask]).ravel()
-                    tn_u, fp_u, fn_u, tp_u  = confusion_matrix(np.array(y_true)[~mask], np.array(y_pred)[~mask]).ravel()
+                    tn_p, fp_p, fn_p, tp_p = confusion_matrix(y_true=np.array(y_true)[mask], y_pred=np.array(y_pred)[mask]).ravel()
+                    tn_u, fp_u, fn_u, tp_u  = confusion_matrix(y_true=np.array(y_true)[~mask], y_pred=np.array(y_pred)[~mask]).ravel()
                 else :
-                    tn_p, fp_p, fn_p, tp_p = confusion_matrix(np.array(y_true)[mask], np.array(y_pred)[mask], sample_weight = sample_weight[mask]).ravel()
-                    tn_u, fp_u, fn_u, tp_u  = confusion_matrix(np.array(y_true)[~mask], np.array(y_pred)[~mask], sample_weight = sample_weight[~mask]).ravel()
+                    tn_p, fp_p, fn_p, tp_p = confusion_matrix(y_true=np.array(y_true)[mask], y_pred=np.array(y_pred)[mask], sample_weight = sample_weight[mask]).ravel()
+                    tn_u, fp_u, fn_u, tp_u  = confusion_matrix(y_true=np.array(y_true)[~mask], y_pred=np.array(y_pred)[~mask], sample_weight = sample_weight[~mask]).ravel()
     
                 if self._rejection_inference_flag[curr_p_var] == True :
                     fn_p = self.spl_params['num_applicants'][curr_p_var][0] * (1-self.spl_params['base_default_rate'][curr_p_var][0]) - tp_p
@@ -661,4 +711,4 @@ class CreditScoring(Fairness):
                 return [None] * 4
                 
             else :
-                return [None] * 8
\ No newline at end of file
+                return [None] * 8
