@@ -2,7 +2,7 @@ import numpy as np
 from sklearn.metrics import confusion_matrix
 from .fairness import Fairness
 from ..utility import check_datatype, check_value
-from .performance_metrics import PerformanceMetrics
+from ..performance_metrics import PerformanceMetrics
 from .fairness_metrics import FairnessMetrics
 from ..config.constants import Constants
 from ..ErrorCatcher import VeritasError
@@ -14,16 +14,15 @@ class CustomerMarketing(Fairness):
     Class Attributes
     ------------------
     _model_type_to_metric_lookup: dictionary
-                Used to associate the model type (key) with the metric type & expected size of positive and negative labels (value).
-                e.g. {“rejection”: (“classification”, 2), “uplift”: (“uplift”, 4), “a_new_type”: (“regression”, -1)}
+                Used to associate the model type (key) with the metric type, expected size of positive and negative labels (value) & length of model_params respectively.
+                e.g. {“rejection”: (“classification”, 2, 1), “uplift”: (“uplift”, 4, 2), “a_new_type”: (“regression”, -1, 1)}
 
     """
-
     _model_type_to_metric_lookup = {"uplift":("uplift", 4, 2),
                                    "rejection": ("classification", 2, 1),
                                    "propensity":("classification", 2, 1)}
 
-    def __init__(self, model_params, fair_threshold, perf_metric_name =  "balanced_acc", fair_metric_name = "auto",  fair_concern = "eligible", fair_priority = "benefit", fair_impact = "normal", fair_neutral_tolerance = 0.002, treatment_cost = None, revenue = None, fairness_metric_value_input = {}, proportion_of_interpolation_fitting = 1.0):
+    def __init__(self, model_params, fair_threshold, perf_metric_name =  "balanced_acc", fair_metric_name = "auto",  fair_concern = "eligible", fair_priority = "benefit", fair_impact = "normal", treatment_cost = None, revenue = None, fairness_metric_value_input = {}, proportion_of_interpolation_fitting = 1.0):
         """
         Parameters
         ----------
@@ -54,7 +53,7 @@ class CustomerMarketing(Fairness):
         fair_impact: string, default = "normal"
                 Used to pick the fairness metric according to the Fairness Tree methodology. Could be "normal" or "significant" or "selective"
 
-        fair_neutral_tolerance: float, default = 0.002
+        fair_neutral_tolerance: float, default = 0.001
                 Tolerance value between 0 and 0.1 (inclusive) used in the performance-fairness tradeoff analysis section to filter the primary fairness metric values.
 
         treatment_cost: float, default=None
@@ -65,7 +64,7 @@ class CustomerMarketing(Fairness):
 
         fairness_metric_value_input : dictionary
 
-        proportion_of_interpolation_fitting : float
+        proportion_of_interpolation_fitting : float, default=1.0
 
         _use_case_metrics: dictionary of lists, default="None"
                 Contains all the performance & fairness metrics for credit scoring.
@@ -99,9 +98,11 @@ class CustomerMarketing(Fairness):
         super().__init__(model_params)
 
         self.fair_metric_name = fair_metric_name
+        self.fair_metric_input = fair_metric_name
         self.perf_metric_name = perf_metric_name
         self.fair_threshold = fair_threshold
-        self.fair_neutral_tolerance = fair_neutral_tolerance
+        self.fair_threshold_input = fair_threshold
+        self.fair_neutral_tolerance = Constants().fair_neutral_tolerance 
         self.fair_concern = fair_concern
         self.fair_priority = fair_priority
         self.fair_impact = fair_impact
@@ -134,11 +135,11 @@ class CustomerMarketing(Fairness):
             "fair_threshold": [(float, int), (Constants().fair_threshold_low), Constants().fair_threshold_high],
             "fair_neutral_tolerance": [(float,),(Constants().fair_neutral_threshold_low), Constants().fair_neutral_threshold_high],
             "proportion_of_interpolation_fitting": [(float,), (Constants().proportion_of_interpolation_fitting_low), Constants().proportion_of_interpolation_fitting_high],
-            "perf_metric_name": [(str,), self._use_case_metrics["perf"]],
-            "fair_metric_name": [(str,), self._use_case_metrics["fair"]],
             "fair_concern": [(str,), ["eligible", "inclusive", "both"]],
             "fair_priority": [(str,), ["benefit", "harm"]],
             "fair_impact": [(str,), ["normal", "significant", "selective"]],
+            "perf_metric_name": [(str,), self._use_case_metrics["perf"]],
+            "fair_metric_name": [(str,), self._use_case_metrics["fair"]],
             "model_params":[(list,), None],
             "fairness_metric_value_input":[(dict,), None]}
 
@@ -146,7 +147,8 @@ class CustomerMarketing(Fairness):
         self.array_size = Constants().array_size
         self.decimals = Constants().decimals
 
-        self.spl_params = {'revenue': revenue, 'treatment_cost': treatment_cost}
+        if self.model_params[0].model_type == "uplift":
+            self.spl_params = {'revenue': revenue, 'treatment_cost': treatment_cost}
         self.selection_threshold = Constants().selection_threshold
 
         self._check_input()
@@ -178,7 +180,7 @@ class CustomerMarketing(Fairness):
         # Check for length of model_params
         # mp_errmsg = "\n The given length of model_params is {}. \n The expected length of model_params is {}."
         mp_g = len(self.model_params)
-        mp_e = int(self._model_type_to_metric_lookup[self.model_params[0].model_type][1] / 2)
+        mp_e = self._model_type_to_metric_lookup[self.model_params[0].model_type][2]
         if mp_g != mp_e:
             err.push('length_error', var_name="model_params", given=str(mp_g), expected= str(mp_e), function_name="_check_input")
 
@@ -202,13 +204,13 @@ class CustomerMarketing(Fairness):
             #y_prob_errmsg = "\n The given dtype of y_prob is {}. \n The expected dtype of y_prob is {}."
             for i in range(len(self.model_params)):
                 if self.model_params[i].y_prob is None:
-                    self.err.push('type_error', var_name="y_prob", given= "type None", expected="type [list, np.ndarray, pd.Series]")
+                    err.push('type_error', var_name="y_prob", given= "type None", expected="type [list, np.ndarray, pd.Series]", function_name="_check_input")
                     #raise TypeError(y_prob_errmsg.format(str(y_prob_given), str(y_prob_expected)))
         else:
             #y_pred_errmsg = "\n The given dtype of y_pred is {}. \n The expected dtype of y_pred is {}."
             for i in range(len(self.model_params)):
                 if self.model_params[i].y_pred is None:
-                    self.err.push('type_error', var_name="y_pred", given= "type None", expected="type [list, np.ndarray, pd.Series]")
+                    err.push('type_error', var_name="y_pred", given= "type None", expected="type [list, np.ndarray, pd.Series]", function_name="_check_input")
                     #raise TypeError(y_pred_errmsg.format(str(y_pred_given), str(y_pred_expected)))
 
          #check if y_pred is provided but model_type is uplift, set y_pred = None
@@ -220,7 +222,7 @@ class CustomerMarketing(Fairness):
 
         #Check for revenue and treatment_cost
         # r_tc_less = "\n The given value of revenue is {} and treatment_cost is {}. \n The expected value of revenue cannot be less than treatment_cost."
-        if 1 == 1:
+        if self.model_params[0].model_type == 'uplift' and self.perf_metric_name == "expected_profit":
         #if self.model_params[0].model_type == 'uplift':
             # errMsg = "data type error"
             # errMsgFormat = "\n    {}: given {}, expected {}"
@@ -236,18 +238,21 @@ class CustomerMarketing(Fairness):
             # errMsg = "data value error"
             # errMsgFormat = "\n    {}: given {:.{decimal_pts}}, expected in range {}"
             # spl_range = (0, np.inf)
-                if type(self.spl_params[i]) != type(None):
-                    if  self.spl_params[i] < spl_range[0]  or self.spl_params[i] > spl_range[1] :
-                    # errMsg += errMsgFormat.format(i, self.spl_params[i], str(spl_range), decimal_pts=self.decimals)
-                    # raise ValueError(errMsg)
-                        err.push('value_error', var_name=str(i), given=self.spl_params[i],  expected="range " + str(spl_range), function_name="_check_input")
+                try:
+                    if type(self.spl_params[i]) != type(None):
+                        if  self.spl_params[i] < spl_range[0]  or self.spl_params[i] > spl_range[1] :
+                        # errMsg += errMsgFormat.format(i, self.spl_params[i], str(spl_range), decimal_pts=self.decimals)
+                        # raise ValueError(errMsg)
+                            err.push('value_error', var_name=str(i), given=self.spl_params[i],  expected="range " + str(spl_range), function_name="_check_input")
+                except:
+                    pass
             try:
                 if self.spl_params['revenue'] < self.spl_params['treatment_cost']:
                     # raise ValueError (r_tc_less.format(str(self.spl_params['revenue']), str(self.spl_params['treatment_cost'])))
                     err.push('value_error_compare', var_name_a="revenue", var_name_b="treatment_cost", function_name="_check_input")
             except:
                 pass
-        
+
         err.pop()
     
     def _get_confusion_matrix(self, y_true, y_pred, sample_weight, curr_p_var = None, feature_mask = None, **kwargs):
@@ -284,25 +289,31 @@ class CustomerMarketing(Fairness):
             if 'y_pred' in kwargs:
                 y_pred = kwargs['y_pred']
             
-            if curr_p_var == None :
-                if sample_weight == None :
+            if curr_p_var is None :
+                if y_pred is None:
+                    return [None] * 4
+                
+                if sample_weight is None :
                     tn, fp, fn, tp = confusion_matrix(y_true, y_pred).ravel()
                 else :
-                    tn, fp, fn, tp = confusion_matrix(y_true, y_pred, sample_weight).ravel()
+                    tn, fp, fn, tp = confusion_matrix(y_true, y_pred, sample_weight = sample_weight).ravel()
                 return tp, fp, tn, fn 
             
             else:
+                if y_pred is None:
+                    return [None] * 8
+                
                 mask = feature_mask[curr_p_var]
-                if sample_weight == None :
+                if sample_weight is None :
                     mask = self.feature_mask[curr_p_var]
                     tn_p, fp_p, fn_p, tp_p = confusion_matrix(np.array(y_true)[mask], np.array(y_pred)[mask]).ravel()
                     tn_u, fp_u, fn_u, tp_u  = confusion_matrix(np.array(y_true)[~mask], np.array(y_pred)[~mask]).ravel()
                 else :
-                    tn_p, fp_p, fn_p, tp_p = confusion_matrix(np.array(y_true)[mask], np.array(y_pred)[mask], sample_weight[mask]).ravel()
-                    tn_u, fp_u, fn_u, tp_u  = confusion_matrix(np.array(y_true)[~mask], np.array(y_pred)[~mask], sample_weight[~mask]).ravel()
+                    tn_p, fp_p, fn_p, tp_p = confusion_matrix(np.array(y_true)[mask], np.array(y_pred)[mask], sample_weight = sample_weight[mask]).ravel()
+                    tn_u, fp_u, fn_u, tp_u  = confusion_matrix(np.array(y_true)[~mask], np.array(y_pred)[~mask], sample_weight = sample_weight[~mask]).ravel()
                 return tp_p, fp_p, tn_p, fn_p, tp_u, fp_u, tn_u, fn_u
         else :
-            if curr_p_var == None:
+            if curr_p_var is None:
                 return [None] * 4
     
             else:
@@ -330,6 +341,19 @@ class CustomerMarketing(Fairness):
                 self._fairness_tree(is_pos_label_favourable)
         else :
             self.fair_metric_name
+        # try:
+        if FairnessMetrics.map_fair_metric_to_group[self.fair_metric_name][4] == False:
+            err = VeritasError()
+            ratio_parity_metrics = []
+            for i,j in FairnessMetrics.map_fair_metric_to_group.items():
+                if j[1] == self._model_type_to_metric_lookup[self.model_params[0].model_type][0]:
+                    if j[2] == "ratio" or j[2] == "parity":
+                        ratio_parity_metrics.append(i)
+            err.push('value_error', var_name="fair_metric_name", given=self.fair_metric_name,  expected=ratio_parity_metrics, function_name="_select_fairness_metric_name")
+            err.pop()
+        # except:
+            # pass
+
 
     def _get_e_lift(self, **kwargs):
         """
@@ -381,6 +405,7 @@ class CustomerMarketing(Fairness):
         Returns
         -----------
         pred_outcome : dictionary
+                Contains the probabilities of the treatment and control groups for both rejection and acquiring
         """
 
         if self.model_params[0].model_type == 'uplift':
@@ -416,4 +441,4 @@ class CustomerMarketing(Fairness):
         else :
             return None
 
-            
\ No newline at end of file
+            
